"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@project-serum/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const serum3_1 = require("../accounts/serum3");
const client_1 = require("../client");
const constants_1 = require("../constants");
//
// script which shows how to close a mango account cleanly i.e. close all active positions, withdraw all tokens, etc.
//
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
// note: either use finalized or expect closing certain things to fail and having to runs scrript multiple times
async function main() {
    var _a, _b, _c, _d;
    const options = anchor_1.AnchorProvider.defaultOptions();
    // note: see note above
    // options.commitment = 'finalized';
    const connection = new web3_js_1.Connection('https://mango.devnet.rpcpool.com', options);
    // user
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.USER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, 'devnet', constants_1.MANGO_V4_ID['devnet'], {
        idsSource: 'get-program-accounts',
    });
    console.log(`User ${userWallet.publicKey.toBase58()}`);
    try {
        // fetch group
        const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.ADMIN_KEYPAIR, 'utf-8'))));
        const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
        console.log(`Found group ${group.publicKey.toBase58()}`);
        // fetch account
        const mangoAccount = (await client.getMangoAccountsForOwner(group, user.publicKey))[0];
        console.log(`...found mangoAccount ${mangoAccount.publicKey}`);
        console.log(mangoAccount.toString());
        // close mango account serum3 positions, closing might require cancelling orders and settling
        for (const serum3Account of mangoAccount.serum3Active()) {
            let orders = await mangoAccount.loadSerum3OpenOrdersForMarket(client, group, (_a = group.serum3MarketsMapByMarketIndex.get(serum3Account.marketIndex)) === null || _a === void 0 ? void 0 : _a.serumMarketExternal);
            for (const order of orders) {
                console.log(` - Order orderId ${order.orderId}, ${order.side}, ${order.price}, ${order.size}`);
                console.log(` - Cancelling order with ${order.orderId}`);
                await client.serum3CancelOrder(group, mangoAccount, (_b = group.serum3MarketsMapByMarketIndex.get(serum3Account.marketIndex)) === null || _b === void 0 ? void 0 : _b.serumMarketExternal, order.side === 'buy' ? serum3_1.Serum3Side.bid : serum3_1.Serum3Side.ask, order.orderId);
            }
            await client.serum3SettleFunds(group, mangoAccount, (_c = group.serum3MarketsMapByMarketIndex.get(serum3Account.marketIndex)) === null || _c === void 0 ? void 0 : _c.serumMarketExternal);
            await client.serum3CloseOpenOrders(group, mangoAccount, (_d = group.serum3MarketsMapByMarketIndex.get(serum3Account.marketIndex)) === null || _d === void 0 ? void 0 : _d.serumMarketExternal);
        }
        // we closed a serum account, this changes the health accounts we are passing in for future ixs
        await mangoAccount.reload(client);
        // withdraw all tokens
        for (const token of mangoAccount.tokensActive()) {
            let native = token.balance(group.getFirstBankByTokenIndex(token.tokenIndex));
            // to avoid rounding issues
            if (native.toNumber() < 1) {
                continue;
            }
            let nativeFlooredNumber = Math.floor(native.toNumber());
            console.log(`withdrawing token ${group.getFirstBankByTokenIndex(token.tokenIndex).name} native amount ${nativeFlooredNumber} `);
            const withdrawIx = await client.tokenWithdrawNativeIx(group, mangoAccount, group.getFirstBankByTokenIndex(token.tokenIndex).mint, new anchor_1.BN(nativeFlooredNumber - 1) /* see comment in token_withdraw in program */, false);
            await this.sendAndConfirmTransaction([...withdrawIx], group.addressLookupTablesList);
        }
        // reload and print current positions
        await mangoAccount.reload(client);
        console.log(`...mangoAccount ${mangoAccount.publicKey}`);
        console.log(mangoAccount.toString());
        // close account
        console.log(`Close mango account...`);
        const res = await client.closeMangoAccount(group, mangoAccount);
    }
    catch (error) {
        console.log(error);
    }
    process.exit();
}
main();
//# sourceMappingURL=devnet-user-close-account.js.map